// Copyright (C) 2012 Stefan Prey
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 
// 
// This file is part of kanonprogramme.
// 
// kanonprogramme is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// kanonprogramme is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with kanonprogramme.  If not, see <http://www.gnu.org/licenses/>.
// 
// Diese Datei ist Teil von kanonprogramme.
// 
// kanonprogramme ist Freie Software: Sie koennen es unter den Bedingungen
// der GNU General Public License, wie von der Free Software Foundation,
// Version 3 der Lizenz oder (nach Ihrer Option) jeder spaeteren
// veroeffentlichten Version, weiterverbreiten und/oder modifizieren.
// 
// kanonprogramme wird in der Hoffnung, dass es nuetzlich sein wird, aber
// OHNE JEDE GEWAEHELEISTUNG, bereitgestellt; sogar ohne die implizite
// Gewaehrleistung der MARKTFAEHIGKEIT oder EIGNUNG FUER EINEN BESTIMMTEN ZWECK.
// Siehe die GNU General Public License fuer weitere Details.
// 
// Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
// Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.

#include <iostream>
#include <cstdlib>
using namespace std;

#include "klangfolge.h"

klangfolge::klangfolge(int laenge, int* klaenge, string name)
{
    m_laenge = laenge;
    m_name = name;
    m_folge = new klang[m_laenge];
    if(m_folge == NULL)
    {
	cout << "Kein Speicher" << endl;
	exit(1);
    }
    for(int i = 0; i < m_laenge; i++)
	m_folge[i] = klang(klaenge[i]);
}

klangfolge::klangfolge(int laenge, klang* klaenge, string name)
{
    m_laenge = laenge;
    m_name = name;
    m_folge = new klang[m_laenge];
    if(m_folge == NULL)
    {
	cout << "Kein Speicher" << endl;
	exit(1);
    }
    for(int i = 0; i < m_laenge; i++)
	m_folge[i] = klaenge[i];
}

klangfolge::klangfolge(const klangfolge& ausgangsfolge)
{
    m_folge = new klang[ausgangsfolge.m_laenge];
    if(m_folge == NULL)
    {
	cout << "Kein Speicher" << endl;
	exit(1);
    }
    m_laenge = ausgangsfolge.m_laenge;
    m_name = ausgangsfolge.m_name;
    for(int i = 0; i < m_laenge; i++)
	m_folge[i] = ausgangsfolge.m_folge[i];
}

klangfolge& klangfolge::operator=(const klangfolge& rechter_operand)
{
    if(&rechter_operand == this)
	return *this;
    delete[] m_folge;
    m_folge = new klang[rechter_operand.m_laenge];
    if(m_folge == NULL)
    {
	cout << "Kein Speicher" << endl;
	exit(1);
    }
    for(int i = 0; i < m_laenge; i++)
	m_folge[i] = rechter_operand.m_folge[i];
    m_name = rechter_operand.m_name;
    return *this;
}

klangfolge::~klangfolge()
{
    delete[] m_folge;
}

klangfolge& klangfolge::operator++()
{
    plusplus();
    return *this;
}

klangfolge& klangfolge::operator++(int)
{
    plusplus();
    return *this;
}

klangfolge klangfolge::schnitt(const klangfolge& rechter_operand)
{
    klang* zwischenwert = new klang[m_laenge];
    if(zwischenwert == NULL)
    {
	cout << "Kein Speicher" << endl;
	exit(1);
    }
    for(int i = 0; i < m_laenge; i++)
	zwischenwert[i] = m_folge[i].schnittmenge(rechter_operand.m_folge[i]);
    klangfolge zwischenfolge(m_laenge, zwischenwert);
    delete[] zwischenwert;
    return zwischenfolge;
}

int klangfolge::passt_zu(const klangfolge& rechter_operand)
{
    int passt = 1;
    klangfolge durchschnitt = schnitt(rechter_operand);
    for(int i = 0; i < m_laenge && passt; i++)
	if(!durchschnitt.m_folge[i].gib_wert())
	    passt = 0;
    return passt;
}

void klangfolge::teste_kombinationen_mit(ostream& file, const klangfolge& rechter_operand)
{
    file << "Teste Kombinationen von " << m_name << " und " << rechter_operand.m_name << endl;
    klangfolge zum_versetzen(rechter_operand);
    for(int j = 0; j < 7; j++)
    {
	if(passt_zu(zum_versetzen))
	    file << "Passt bei einer Versetzung von " << j << endl;
	zum_versetzen++;
    }
    file << endl;
}

void klangfolge::plusplus()
{
    for(int i = 0; i < m_laenge; i++)
	m_folge[i].versetze(1);
}
